#########################################################################
#
#
# Copyright 2021-2024: all rights reserved
#
# Author: Dr Giorgio Pioda
# email: giorgio,pioda@spse.ch, giorgio.pioda@edu.ti.ch, gfwp@ticino.com
#
# 
########################################################################

library(parallel)
library(mcr)
library(rrcov)

#Optimum 5 18 24 CPU (71 points)

# Data set generator

cs.st<-function(xmin,xmax,n=40,slope=1,intercept=0,sigmax=1,sigmay=1){
  xr<-runif(n,xmin,xmax)
  yr<-slope*xr+intercept
  xe<-rnorm(1:length(xr),mean=0,sd=sigmax)
  ye<-rnorm(1:length(yr),mean=0,sd=sigmay)
  detmin<-0.1
  X<-ifelse((xr+xe) > detmin, signif(xr+xe,2),detmin/2)
  Y<-ifelse((yr+ye) > detmin, signif(yr+ye,2),detmin/2)
  return(data.frame(cbind(X=X,Y=Y)))
}


# Regression execution and robust covariance determination
# a df is returned with CI and MD (and also other stuff like chisq of MD)
# In this case some covariance/MD is disabled because at 2x2 resolution
# boostrap with equivariant PaBa seems not to be stable enough

mmcombi.st4f<-function(ndat,slope){
  dat<-cs.st(3,8,ndat,slope,0,0.2,0.2)
  p.mcr<-mcreg(dat$X,dat$Y,method.reg = "PaBa", method.ci = "bootstrap",method.bootstrap.ci = "BCa")
  pq.mcr<-mcreg(dat$X,dat$Y,method.reg = "PaBa", method.ci = "bootstrap",method.bootstrap.ci = "quantile")
  ps.mcr<-mcreg(dat$X,dat$Y,method.reg = "PaBa", method.ci = "bootstrap",method.bootstrap.ci = "Student")
  pa.mcr<-mcreg(dat$X,dat$Y,method.reg = "PBequi", method.ci = "analytical")
  pf.mcr<-mcreg(dat$X,dat$Y,method.reg = "PBequi",methodlarge=F, method.ci = "analytical")
  d.mcr<-mcreg(dat$X,dat$Y,method.reg = "Deming",method.ci = "bootstrap",method.bootstrap.ci = "BCa")
  pc.mcr<-mcreg(dat$X,dat$Y,method.reg = "PaBa",method.ci = "bootstrap",method.bootstrap.ci = "BCa")
  po.mcr<-mcreg(dat$X,dat$Y,method.reg = "PaBa", method.ci = "analytical")

  
  p.res<-data.frame(MCResult.getCoefficients(p.mcr))
  pq.res<-data.frame(MCResult.getCoefficients(pq.mcr))
  ps.res<-data.frame(MCResult.getCoefficients(ps.mcr))
  pa.res<-data.frame(MCResult.getCoefficients(pa.mcr))
  pf.res<-data.frame(MCResult.getCoefficients(pf.mcr))
  d.res<-data.frame(MCResult.getCoefficients(d.mcr))
  pc.res<-data.frame(MCResult.getCoefficients(pf.mcr))
  po.res<-data.frame(MCResult.getCoefficients(po.mcr))
  
  #w.res<-data.frame(MCResult.getCoefficients(w.mcr))
  #m.res<-data.frame(MCResult.getCoefficients(m.mcr))
  #mm.res<-data.frame(MCResult.getCoefficients(mm.mcr))
  
  #p.cov<-CovRobust(cbind(p.mcr@B0,p.mcr@B1))
  p.cov<-1
  #pq.cov<-CovRobust(cbind(pq.mcr@B0,pq.mcr@B1))
  pq.cov<-1
  #ps.cov<-CovRobust(cbind(ps.mcr@B0,ps.mcr@B1))
  ps.cov<-1
  pa.cov<-1
  d.cov<-CovRobust(cbind(d.mcr@B0,d.mcr@B1))
  #pc.cov<-CovRobust(cbind(pc.mcr@B0,pc.mcr@B1))
  pc.cov<-1
  po.cov<-1
  
  #w.cov<-CovRobust(cbind(w.mcr@B0,w.mcr@B1))
  #m.cov<-CovRobust(cbind(m.mcr@B0,m.mcr@B1))
  #mm.cov<-CovRobust(cbind(mm.mcr@B0,mm.mcr@B1))
  
  #pmcd.cov<-CovMcd(cbind(p.mcr@B0,p.mcr@B1))
  pmcd.cov<-1
  #pmcdq.cov<-CovMcd(cbind(pq.mcr@B0,pq.mcr@B1))
  pmcdq.cov<-1
  #pmcds.cov<-CovMcd(cbind(ps.mcr@B0,ps.mcr@B1))
  pmcds.cov<-1
  dmcd.cov<-CovMcd(cbind(d.mcr@B0,d.mcr@B1))
  #pmcdc.cov<-CovMcd(cbind(pf.mcr@B0,pf.mcr@B1))
  pmcdc.cov<-1
  
  pmcdf.cov<-1
  pmcdo.cov<-1
  
  #wmcd.cov<-CovMcd(cbind(w.mcr@B0,w.mcr@B1))
  #mmcd.cov<-CovMcd(cbind(m.mcr@B0,m.mcr@B1))
  #mmmcd.cov<-CovMcd(cbind(mm.mcr@B0,mm.mcr@B1))
  
  pcl.cov<-CovClassic(cbind(p.mcr@B0,p.mcr@B1))
  pqcl.cov<-CovClassic(cbind(pq.mcr@B0,pq.mcr@B1))
  pscl.cov<-CovClassic(cbind(ps.mcr@B0,ps.mcr@B1))
  dcl.cov<-CovClassic(cbind(d.mcr@B0,d.mcr@B1))
  pclc.cov<-CovClassic(cbind(pc.mcr@B0,pc.mcr@B1))
  
  #wcl.cov<-CovClassic(cbind(w.mcr@B0,w.mcr@B1))
  #mcl.cov<-CovClassic(cbind(m.mcr@B0,m.mcr@B1))
  #mmcl.cov<-CovClassic(cbind(mm.mcr@B0,mm.mcr@B1))
  
  #p.md<-mahalanobis(c(0,1),center=p.cov$center,cov=p.cov$cov)
  p.md<-0
  #pq.md<-mahalanobis(c(0,1),center=pq.cov$center,cov=pq.cov$cov)
  pq.md<-0
  #ps.md<-mahalanobis(c(0,1),center=ps.cov$center,cov=ps.cov$cov)
  ps.md<-0
  d.md<-mahalanobis(c(0,1),center=d.cov$center,cov=d.cov$cov)
  #pf.md<-mahalanobis(c(0,1),center=pf.cov$center,cov=pf.cov$cov)
  pc.md<-0
  pf.md<-0
  po.md<-0
  
  #w.md<-mahalanobis(c(0,1),center=w.cov$center,cov=w.cov$cov)
  #m.md<-mahalanobis(c(0,1),center=m.cov$center,cov=m.cov$cov)
  #mm.md<-mahalanobis(c(0,1),center=mm.cov$center,cov=mm.cov$cov)
  
  
  #pmcd.md<-mahalanobis(c(0,1),center=pmcd.cov$center,cov=pmcd.cov$cov)
  pmcd.md<-0
  #pmcdq.md<-mahalanobis(c(0,1),center=pmcdq.cov$center,cov=pmcdq.cov$cov)
  pmcdq.md<-0
  #pmcds.md<-mahalanobis(c(0,1),center=pmcds.cov$center,cov=pmcds.cov$cov)
  pmcds.md<-0
  dmcd.md<-mahalanobis(c(0,1),center=dmcd.cov$center,cov=dmcd.cov$cov)
  #pmcdc.md<-mahalanobis(c(0,1),center=pmcdc.cov$center,cov=pmcdc.cov$cov)
  pmcdc.md<-0
  pmcdf.md<-0
  pmcdo.md<-0
  
  #wmcd.md<-mahalanobis(c(0,1),center=wmcd.cov$center,cov=wmcd.cov$cov)
  #mmcd.md<-mahalanobis(c(0,1),center=mmcd.cov$center,cov=mmcd.cov$cov)
  #mmmcd.md<-mahalanobis(c(0,1),center=mmmcd.cov$center,cov=mmmcd.cov$cov)
  
  
  pcl.md<-mahalanobis(c(0,1),center=pcl.cov$center,cov=pcl.cov$cov)
  pqcl.md<-mahalanobis(c(0,1),center=pqcl.cov$center,cov=pqcl.cov$cov)
  pscl.md<-mahalanobis(c(0,1),center=pscl.cov$center,cov=pscl.cov$cov)
  dcl.md<-mahalanobis(c(0,1),center=dcl.cov$center,cov=dcl.cov$cov)
  pclc.md<-mahalanobis(c(0,1),center=pclc.cov$center,cov=pclc.cov$cov)
  
  #wcl.md<-mahalanobis(c(0,1),center=wcl.cov$center,cov=wcl.cov$cov)
  #mcl.md<-mahalanobis(c(0,1),center=mcl.cov$center,cov=mcl.cov$cov)
  #mmcl.md<-mahalanobis(c(0,1),center=mmcl.cov$center,cov=mmcl.cov$cov)
  
  p.chi<-pchisq(p.md,df=2,lower.tail=F)
  pq.chi<-pchisq(pq.md,df=2,lower.tail=F)
  ps.chi<-pchisq(ps.md,df=2,lower.tail=F)
  pa.chi<-1
  pf.chi<-1
  po.chi<-1
  d.chi<-pchisq(d.md,df=2,lower.tail=F)
  pc.chi<-pchisq(pc.md,df=2,lower.tail=F)
  
  #w.chi<-pchisq(w.md,df=2,lower.tail=F)
  #m.chi<-pchisq(m.md,df=2,lower.tail=F)
  #mm.chi<-pchisq(mm.md,df=2,lower.tail=F)
  
  pmcd.chi<-pchisq(pmcd.md,df=2,lower.tail=F)
  pmcdq.chi<-pchisq(pmcdq.md,df=2,lower.tail=F)
  pmcds.chi<-pchisq(pmcds.md,df=2,lower.tail=F)
  pmcda.chi<-1
  pmcdf.chi<-1
  pmcdo.chi<-1
  dmcd.chi<-pchisq(dmcd.md,df=2,lower.tail=F)
  pmcdc.chi<-pchisq(pmcdc.md,df=2,lower.tail=F)
  
  #wmcd.chi<-pchisq(wmcd.md,df=2,lower.tail=F)
  #mmcd.chi<-pchisq(mmcd.md,df=2,lower.tail=F)
  #mmmcd.chi<-pchisq(mmmcd.md,df=2,lower.tail=F)
  
  pcl.chi<-pchisq(pcl.md,df=2,lower.tail=F)
  pqcl.chi<-pchisq(pqcl.md,df=2,lower.tail=F)
  pscl.chi<-pchisq(pscl.md,df=2,lower.tail=F)
  pacl.chi<-1
  pfcl.chi<-1
  pocl.chi<-1
  dcl.chi<-pchisq(dcl.md,df=2,lower.tail=F)
  pclc.chi<-pchisq(pclc.md,df=2,lower.tail=F)
  
  #wcl.chi<-pchisq(wcl.md,df=2,lower.tail=F)
  #mcl.chi<-pchisq(mcl.md,df=2,lower.tail=F)
  #mmcl.chi<-pchisq(mmcl.md,df=2,lower.tail=F)
  
  
  return(c(p.res[1,1],p.res[1,3],p.res[1,4],p.res[2,1],p.res[2,3],p.res[2,4],p.chi,pcl.chi,pmcd.chi,
           d.res[1,1],d.res[1,3],d.res[1,4],d.res[2,1],d.res[2,3],d.res[2,4],d.chi,dcl.chi,dmcd.chi,
           #w.res[1,1],w.res[1,3],w.res[1,4],w.res[2,1],w.res[2,3],w.res[2,4],w.chi,wcl.chi,wmcd.chi,
           #m.res[1,1],m.res[1,3],m.res[1,4],m.res[2,1],m.res[2,3],m.res[2,4],m.chi,mcl.chi,mmcd.chi,
           #mm.res[1,1],mm.res[1,3],mm.res[1,4],mm.res[2,1],mm.res[2,3],mm.res[2,4],mm.chi,mmcl.chi,mmmcd.chi,
           pq.res[1,1],pq.res[1,3],pq.res[1,4],pq.res[2,1],pq.res[2,3],pq.res[2,4],pq.chi,pqcl.chi,pmcdq.chi,
           ps.res[1,1],ps.res[1,3],ps.res[1,4],ps.res[2,1],ps.res[2,3],ps.res[2,4],ps.chi,pscl.chi,pmcds.chi,
           pa.res[1,1],pa.res[1,3],pa.res[1,4],pa.res[2,1],pa.res[2,3],pa.res[2,4],pa.chi,pacl.chi,pmcda.chi,
           pf.res[1,1],pf.res[1,3],pf.res[1,4],pf.res[2,1],pf.res[2,3],pf.res[2,4],pf.chi,pclc.chi,pmcdc.chi,
           pf.res[1,1],pf.res[1,3],pf.res[1,4],pf.res[2,1],pf.res[2,3],pf.res[2,4],pf.chi,pfcl.chi,pmcdf.chi,
           po.res[1,1],po.res[1,3],po.res[1,4],po.res[2,1],po.res[2,3],po.res[2,4],po.chi,pocl.chi,pmcdo.chi
           
           ))
}

# Core checking for parallel computing

numCores<-detectCores()
numCores

# Setting up the slope values

s100.val<-c(seq(0.88,0.925,by=1/200),
            seq(0.93,0.96,by=1/300),
            seq(0.9625,1.0375,by=0.0025),
            seq(1.04,1.07,by=1/300),
            seq(1.075,1.12,by=1/200)
)

length(s100.val)

# For each slope calls the regression function nrep times and then call the proportion counter function.
# This functions is called from mclapply

mmcombi.100st2vs2equi<-function(slope){
  nrep<-250
  r.res<-matrix(NA,ncol=72,nrow=nrep)
  for(i in 1:nrep){
    r.res[i,]<-mmcombi.st4f(ndat=100,slope)
  }
  return(mmrcombi.ck(r.res))
}

# Proportion counter function. Counts how many time the H0 is outside the stored CI, both
# for classical CI and for MD. Returns a data frame of the proportion.

mmrcombi.ck<-function(sim.res){
  p.e500<-mean(sim.res[,7] >= 0.05)
  p.e250<-mean(sim.res[,7] >= 0.025)
  p.e100<-mean(sim.res[,7] >= 0.01)
  p.e050<-mean(sim.res[,7] >= 0.005)
  p.e025<-mean(sim.res[,7] >= 0.0025)
  p.e010<-mean(sim.res[,7] >= 0.0001)
  
  pcl.e500<-mean(sim.res[,8] >= 0.05)
  pcl.e100<-mean(sim.res[,8] >= 0.01)
  
  pmcd.e500<-mean(sim.res[,9] >= 0.05)
  pmcd.e250<-mean(sim.res[,9] >= 0.025)
  pmcd.e100<-mean(sim.res[,9] >= 0.01)
  pmcd.e050<-mean(sim.res[,9] >= 0.005)
  pmcd.e025<-mean(sim.res[,9] >= 0.0025)
  pmcd.e010<-mean(sim.res[,9] >= 0.0001)
  
  p.i<-mean((sim.res[,2]<=0 & sim.res[,3]>=0))
  p.s<-mean((sim.res[,5]<=1 & sim.res[,6]>=1))
  p.c<-mean(((sim.res[,2]<=0 & sim.res[,3]>=0) & (sim.res[,5]<=1 & sim.res[,6]>=1)))
  
  d.e500<-mean(sim.res[,16] >= 0.05)
  d.e250<-mean(sim.res[,16] >= 0.025)
  d.e100<-mean(sim.res[,16] >= 0.01)
  d.e050<-mean(sim.res[,16] >= 0.005)
  d.e025<-mean(sim.res[,16] >= 0.0025)
  d.e010<-mean(sim.res[,16] >= 0.0001)
  
  dcl.e500<-mean(sim.res[,17] >= 0.05)
  dcl.e100<-mean(sim.res[,17] >= 0.01)
  
  dmcd.e500<-mean(sim.res[,18] >= 0.05)
  dmcd.e250<-mean(sim.res[,18] >= 0.025)
  dmcd.e100<-mean(sim.res[,18] >= 0.01)
  dmcd.e050<-mean(sim.res[,18] >= 0.005)
  dmcd.e025<-mean(sim.res[,18] >= 0.0025)
  dmcd.e010<-mean(sim.res[,18] >= 0.0001)
  
  d.i<-mean((sim.res[,11]<=0 & sim.res[,12]>=0))
  d.s<-mean((sim.res[,14]<=1 & sim.res[,15]>=1))
  d.c<-mean(((sim.res[,11]<=0 & sim.res[,12]>=0) & (sim.res[,14]<=1 & sim.res[,15]>=1)))
  
  #w.e500<-mean(sim.res[,25] >= 0.05)
  #w.e250<-mean(sim.res[,25] >= 0.025)
  #w.e100<-mean(sim.res[,25] >= 0.01)
  #w.e050<-mean(sim.res[,25] >= 0.005)
  #w.e025<-mean(sim.res[,25] >= 0.0025)
  #w.e010<-mean(sim.res[,25] >= 0.0001)
  
  #wcl.e500<-mean(sim.res[,26] >= 0.05)
  #wcl.e100<-mean(sim.res[,26] >= 0.01)
  
  #wmcd.e500<-mean(sim.res[,27] >= 0.05)
  #wmcd.e250<-mean(sim.res[,27] >= 0.025)
  #wmcd.e100<-mean(sim.res[,27] >= 0.01)
  #wmcd.e050<-mean(sim.res[,27] >= 0.005)
  #wmcd.e025<-mean(sim.res[,27] >= 0.0025)
  #wmcd.e010<-mean(sim.res[,27] >= 0.0001)
  
  #w.i<-mean((sim.res[,20]<=0 & sim.res[,21]>=0))
  #w.s<-mean((sim.res[,23]<=1 & sim.res[,24]>=1))
  #w.c<-mean(((sim.res[,20]<=0 & sim.res[,21]>=0) & (sim.res[,23]<=1 & sim.res[,24]>=1)))
  
  #m.e500<-mean(sim.res[,34] >= 0.05)
  #m.e250<-mean(sim.res[,34] >= 0.025)
  #m.e100<-mean(sim.res[,34] >= 0.01)
  #m.e050<-mean(sim.res[,34] >= 0.005)
  #m.e025<-mean(sim.res[,34] >= 0.0025)
  #m.e010<-mean(sim.res[,34] >= 0.0001)
  
  #mcl.e500<-mean(sim.res[,35] >= 0.05)
  #mcl.e100<-mean(sim.res[,35] >= 0.01)
  
  #mmcd.e500<-mean(sim.res[,36] >= 0.05)
  #mmcd.e250<-mean(sim.res[,36] >= 0.025)
  #mmcd.e100<-mean(sim.res[,36] >= 0.01)
  #mmcd.e050<-mean(sim.res[,36] >= 0.005)
  #mmcd.e025<-mean(sim.res[,36] >= 0.0025)
  #mmcd.e010<-mean(sim.res[,36] >= 0.0001)
  
  #m.i<-mean((sim.res[,29]<=0 & sim.res[,30]>=0))
  #m.s<-mean((sim.res[,32]<=1 & sim.res[,33]>=1))
  #m.c<-mean(((sim.res[,29]<=0 & sim.res[,30]>=0) & (sim.res[,32]<=1 & sim.res[,33]>=1)))
  
  #mm.e500<-mean(sim.res[,43] >= 0.05)
  #mm.e250<-mean(sim.res[,43] >= 0.025)
  #mm.e100<-mean(sim.res[,43] >= 0.01)
  #mm.e050<-mean(sim.res[,43] >= 0.005)
  #mm.e025<-mean(sim.res[,43] >= 0.0025)
  #mm.e010<-mean(sim.res[,43] >= 0.0001)
  
  #mmcl.e500<-mean(sim.res[,44] >= 0.05)
  #mmcl.e100<-mean(sim.res[,44] >= 0.01)
  
  #mmmcd.e500<-mean(sim.res[,45] >= 0.05)
  #mmmcd.e250<-mean(sim.res[,45] >= 0.025)
  #mmmcd.e100<-mean(sim.res[,45] >= 0.01)
  #mmmcd.e050<-mean(sim.res[,45] >= 0.005)
  #mmmcd.e025<-mean(sim.res[,45] >= 0.0025)
  #mmmcd.e010<-mean(sim.res[,45] >= 0.0001)
  
  #mm.i<-mean((sim.res[,38]<=0 & sim.res[,39]>=0))
  #mm.s<-mean((sim.res[,41]<=1 & sim.res[,42]>=1))
  #mm.c<-mean(((sim.res[,38]<=0 & sim.res[,39]>=0) & (sim.res[,41]<=1 & sim.res[,42]>=1)))
  
  pq.e500<-mean(sim.res[,25] >= 0.05)
  pq.e250<-mean(sim.res[,25] >= 0.025)
  pq.e100<-mean(sim.res[,25] >= 0.01)
  pq.e050<-mean(sim.res[,25] >= 0.005)
  pq.e025<-mean(sim.res[,25] >= 0.0025)
  pq.e010<-mean(sim.res[,25] >= 0.0001)
  
  pqcl.e500<-mean(sim.res[,26] >= 0.05)
  pqcl.e100<-mean(sim.res[,26] >= 0.01)
  
  pmcdq.e500<-mean(sim.res[,27] >= 0.05)
  pmcdq.e250<-mean(sim.res[,27] >= 0.025)
  pmcdq.e100<-mean(sim.res[,27] >= 0.01)
  pmcdq.e050<-mean(sim.res[,27] >= 0.005)
  pmcdq.e025<-mean(sim.res[,27] >= 0.0025)
  pmcdq.e010<-mean(sim.res[,27] >= 0.0001)
  
  pq.i<-mean((sim.res[,20]<=0 & sim.res[,21]>=0))
  pq.s<-mean((sim.res[,23]<=1 & sim.res[,24]>=1))
  pq.c<-mean(((sim.res[,20]<=0 & sim.res[,21]>=0) & (sim.res[,23]<=1 & sim.res[,24]>=1)))
  
  ps.e500<-mean(sim.res[,34] >= 0.05)
  ps.e250<-mean(sim.res[,34] >= 0.025)
  ps.e100<-mean(sim.res[,34] >= 0.01)
  ps.e050<-mean(sim.res[,34] >= 0.005)
  ps.e025<-mean(sim.res[,34] >= 0.0025)
  ps.e010<-mean(sim.res[,34] >= 0.0001)
  
  pscl.e500<-mean(sim.res[,35] >= 0.05)
  pscl.e100<-mean(sim.res[,35] >= 0.01)
  
  pmcds.e500<-mean(sim.res[,36] >= 0.05)
  pmcds.e250<-mean(sim.res[,36] >= 0.025)
  pmcds.e100<-mean(sim.res[,36] >= 0.01)
  pmcds.e050<-mean(sim.res[,36] >= 0.005)
  pmcds.e025<-mean(sim.res[,36] >= 0.0025)
  pmcds.e010<-mean(sim.res[,36] >= 0.0001)
  
  ps.i<-mean((sim.res[,29]<=0 & sim.res[,30]>=0))
  ps.s<-mean((sim.res[,32]<=1 & sim.res[,33]>=1))
  ps.c<-mean(((sim.res[,29]<=0 & sim.res[,30]>=0) & (sim.res[,32]<=1 & sim.res[,33]>=1)))
  
  pa.e500<-mean(sim.res[,43] >= 0.05)
  pa.e250<-mean(sim.res[,43] >= 0.025)
  pa.e100<-mean(sim.res[,43] >= 0.01)
  pa.e050<-mean(sim.res[,43] >= 0.005)
  pa.e025<-mean(sim.res[,43] >= 0.0025)
  pa.e010<-mean(sim.res[,43] >= 0.0001)
  
  pacl.e500<-mean(sim.res[,44] >= 0.05)
  pacl.e100<-mean(sim.res[,44] >= 0.01)
  
  pmcda.e500<-mean(sim.res[,45] >= 0.05)
  pmcda.e250<-mean(sim.res[,45] >= 0.025)
  pmcda.e100<-mean(sim.res[,45] >= 0.01)
  pmcda.e050<-mean(sim.res[,45] >= 0.005)
  pmcda.e025<-mean(sim.res[,45] >= 0.0025)
  pmcda.e010<-mean(sim.res[,45] >= 0.0001)
  
  pa.i<-mean((sim.res[,38]<=0 & sim.res[,39]>=0))
  pa.s<-mean((sim.res[,41]<=1 & sim.res[,42]>=1))
  pa.c<-mean(((sim.res[,38]<=0 & sim.res[,39]>=0) & (sim.res[,41]<=1 & sim.res[,42]>=1)))
  
  pc.e500<-mean(sim.res[,52] >= 0.05)
  pc.e250<-mean(sim.res[,52] >= 0.025)
  pc.e100<-mean(sim.res[,52] >= 0.01)
  pc.e050<-mean(sim.res[,52] >= 0.005)
  pc.e025<-mean(sim.res[,52] >= 0.0025)
  pc.e010<-mean(sim.res[,52] >= 0.0001)
  
  pclc.e500<-mean(sim.res[,53] >= 0.05)
  pclc.e100<-mean(sim.res[,53] >= 0.01)
  
  pmcdc.e500<-mean(sim.res[,54] >= 0.05)
  pmcdc.e250<-mean(sim.res[,54] >= 0.025)
  pmcdc.e100<-mean(sim.res[,54] >= 0.01)
  pmcdc.e050<-mean(sim.res[,54] >= 0.005)
  pmcdc.e025<-mean(sim.res[,54] >= 0.0025)
  pmcdc.e010<-mean(sim.res[,54] >= 0.0001)
  
  pc.i<-mean((sim.res[,47]<=0 & sim.res[,48]>=0))
  pc.s<-mean((sim.res[,50]<=1 & sim.res[,51]>=1))
  pc.c<-mean(((sim.res[,47]<=0 & sim.res[,48]>=0) & (sim.res[,50]<=1 & sim.res[,51]>=1)))
  
  pf.e500<-mean(sim.res[,61] >= 0.05)
  pf.e250<-mean(sim.res[,61] >= 0.025)
  pf.e100<-mean(sim.res[,61] >= 0.01)
  pf.e050<-mean(sim.res[,61] >= 0.005)
  pf.e025<-mean(sim.res[,61] >= 0.0025)
  pf.e010<-mean(sim.res[,61] >= 0.0001)
  
  pclf.e500<-mean(sim.res[,62] >= 0.05)
  pclf.e100<-mean(sim.res[,62] >= 0.01)
  
  pmcdf.e500<-mean(sim.res[,63] >= 0.05)
  pmcdf.e250<-mean(sim.res[,63] >= 0.025)
  pmcdf.e100<-mean(sim.res[,63] >= 0.01)
  pmcdf.e050<-mean(sim.res[,63] >= 0.005)
  pmcdf.e025<-mean(sim.res[,63] >= 0.0025)
  pmcdf.e010<-mean(sim.res[,63] >= 0.0001)
  
  pf.i<-mean((sim.res[,56]<=0 & sim.res[,57]>=0))
  pf.s<-mean((sim.res[,59]<=1 & sim.res[,60]>=1))
  pf.c<-mean(((sim.res[,56]<=0 & sim.res[,57]>=0) & (sim.res[,59]<=1 & sim.res[,60]>=1)))
  
  po.e500<-mean(sim.res[,70] >= 0.05)
  po.e250<-mean(sim.res[,70] >= 0.025)
  po.e100<-mean(sim.res[,70] >= 0.01)
  po.e050<-mean(sim.res[,70] >= 0.005)
  po.e025<-mean(sim.res[,70] >= 0.0025)
  po.e010<-mean(sim.res[,70] >= 0.0001)
  
  pclo.e500<-mean(sim.res[,71] >= 0.05)
  pclo.e100<-mean(sim.res[,71] >= 0.01)
  
  pmcdo.e500<-mean(sim.res[,72] >= 0.05)
  pmcdo.e250<-mean(sim.res[,72] >= 0.025)
  pmcdo.e100<-mean(sim.res[,72] >= 0.01)
  pmcdo.e050<-mean(sim.res[,72] >= 0.005)
  pmcdo.e025<-mean(sim.res[,72] >= 0.0025)
  pmcdo.e010<-mean(sim.res[,72] >= 0.0001)
  
  po.i<-mean((sim.res[,65]<=0 & sim.res[,66]>=0))
  po.s<-mean((sim.res[,68]<=1 & sim.res[,69]>=1))
  po.c<-mean(((sim.res[,65]<=0 & sim.res[,66]>=0) & (sim.res[,68]<=1 & sim.res[,69]>=1)))
  
  
  return(c(p.i,p.s,p.c,p.e500,p.e250,p.e100,p.e050,p.e025,p.e010,pcl.e500,pcl.e100,pmcd.e500,pmcd.e250,pmcd.e100,pmcd.e050,pmcd.e025,pmcd.e010,
           d.i,d.s,d.c,d.e500,d.e250,d.e100,d.e050,d.e025,d.e010,dcl.e500,dcl.e100,dmcd.e500,dmcd.e250,dmcd.e100,dmcd.e050,dmcd.e025,dmcd.e010,
           #w.i,w.s,w.c,w.e500,w.e250,w.e100,w.e050,w.e025,w.e010,wcl.e500,wcl.e100,wmcd.e500,wmcd.e250,wmcd.e100,wmcd.e050,wmcd.e025,wmcd.e010,
           #m.i,m.s,m.c,m.e500,m.e250,m.e100,m.e050,m.e025,m.e010,mcl.e500,mcl.e100,mmcd.e500,mmcd.e250,mmcd.e100,mmcd.e050,mmcd.e025,mmcd.e010,
           #mm.i,mm.s,mm.c,mm.e500,mm.e250,mm.e100,mm.e050,mm.e025,mm.e010,mmcl.e500,mmcl.e100,mmmcd.e500,mmmcd.e250,mmmcd.e100,mmmcd.e050,mmmcd.e025,mmmcd.e010,
           pq.i,pq.s,pq.c,pq.e500,pq.e250,pq.e100,pq.e050,pq.e025,pq.e010,pqcl.e500,pqcl.e100,pmcdq.e500,pmcdq.e250,pmcdq.e100,pmcdq.e050,pmcdq.e025,pmcdq.e010,
           ps.i,ps.s,ps.c,ps.e500,ps.e250,ps.e100,ps.e050,ps.e025,ps.e010,pscl.e500,pscl.e100,pmcds.e500,pmcds.e250,pmcds.e100,pmcds.e050,pmcds.e025,pmcds.e010,
           pa.i,pa.s,pa.c,pa.e500,pa.e250,pa.e100,pa.e050,pa.e025,pa.e010,pacl.e500,pacl.e100,pmcda.e500,pmcda.e250,pmcda.e100,pmcda.e050,pmcda.e025,pmcda.e010,
           pc.i,pc.s,pc.c,pc.e500,pc.e250,pc.e100,pc.e050,pc.e025,pc.e010,pclc.e500,pclc.e100,pmcdc.e500,pmcdc.e250,pmcdc.e100,pmcdc.e050,pmcdc.e025,pmcdc.e010,
           pf.i,pf.s,pf.c,pf.e500,pf.e250,pf.e100,pf.e050,pf.e025,pf.e010,pclf.e500,pclf.e100,pmcdf.e500,pmcdf.e250,pmcdf.e100,pmcdf.e050,pmcdf.e025,pmcdf.e010,
           po.i,po.s,po.c,po.e500,po.e250,po.e100,po.e050,po.e025,po.e010,pclo.e500,pclo.e100,pmcdo.e500,pmcdo.e250,pmcdo.e100,pmcdo.e050,pmcdo.e025,pmcdo.e010
           ))
}

# Hard work happens here. Parallelized mclapply produces a list with the proportions

system.time(
  defcomp100st2vs2equiE2.res<-mclapply(s100.val,mmcombi.100st2vs2equi,mc.cores = (numCores))
)

# Reformatting of the list into a single df and renaming

defcomp100st2vs2equiE2.r<-matrix(unlist(defcomp100st2vs2equiE2.res),ncol=136,nrow=length(s100.val),byrow=T)
defcomp100st2vs2equiE2.r<-data.frame(defcomp100st2vs2equiE2.r)
colnames(defcomp100st2vs2equiE2.r)<-c("p.int","p.slope","p.tot","p.e500", "p.e250","p.e100","p.e050","p.e025","p.e010","pcl.e500","pcl.e100","pmcd.e500","pmcd.e250","pmcd.e100","pmcd.e050","pmcd.e025","pmcd.e010",
                           "d.int","d.slope","d.tot","d.e500", "d.e250","d.e100","d.e050","d.e025","d.e010","dcl.e500","dcl.e100","dmcd.e500","dmcd.e250","dmcd.e100","dmcd.e050","dmcd.e025","dmcd.e010",
                           #"w.int","w.slope","w.tot","w.e500", "w.e250","w.e100","w.e050","w.e025","w.e010","wcl.e500","wcl.e100","wmcd.e500","wmcd.e250","wmcd.e100","wmcd.e050","wmcd.e025","wmcd.e010",
                           #"m.int","m.slope","m.tot","m.e500", "m.e250","m.e100","m.e050","m.e025","m.e010","mcl.e500","mcl.e100","mmcd.e500","mmcd.e250","mmcd.e100","mmcd.e050","mmcd.e025","mmcd.e010",
                           #"mm.int","mm.slope","mm.tot","mm.e500", "mm.e250","mm.e100","mm.e050","mm.e025","mm.e010","mmcl.e500","mmcl.e100","mmmcd.e500","mmmcd.e250","mmmcd.e100","mmmcd.e050","mmmcd.e025","mmmcd.e010",
                           "pq.int","pq.slope","pq.tot","pq.e500", "pq.e250","pq.e100","pq.e050","pq.e025","pq.e010","pqcl.e500","pqcl.e100","pmcdq.e500","pmcdq.e250","pmcdq.e100","pmcdq.e050","pmcdq.e025","pmcdq.e010",
                           "ps.int","ps.slope","ps.tot","ps.e500", "ps.e250","ps.e100","ps.e050","ps.e025","ps.e010","pscl.e500","pscl.e100","pmcs.e500","pmcds.e250","pmcds.e100","pmcds.e050","pmcds.e025","pmcds.e010",
                           "pa.int","pa.slope","pa.tot","pa.e500", "pa.e250","pa.e100","pa.e050","pa.e025","pa.e010","pacl.e500","pacl.e100","pmcda.e500","pmcda.e250","pmcda.e100","pmcda.e050","pmcda.e025","pmcda.e010",
                           "pc.int","pc.slope","pc.tot","pc.e500", "pc.e250","pc.e100","pc.e050","pc.e025","pc.e010","pclc.e500","pclc.e100","pmcdc.e500","pmcdc.e250","pmcdc.e100","pmcdc.e050","pmcdc.e025","pmcdc.e010",
                           "pf.int","pf.slope","pf.tot","pf.e500", "pf.e250","pf.e100","pf.e050","pf.e025","pf.e010","pclf.e500","pclf.e100","pmcdf.e500","pmcdf.e250","pmcdf.e100","pmcdf.e050","pmcdf.e025","pmcdf.e010",
                           "po.int","po.slope","po.tot","po.e500", "po.e250","po.e100","po.e050","po.e025","po.e010","pclo.e500","pclo.e100","pmcdo.e500","pmcdo.e250","pmcdo.e100","pmcdo.e050","pmcdo.e025","pmcdo.e010"
                        )

# Add the slope column and store the final df of the results

defcomp100st2vs2equiE2.r$slope<-s100.val
write.csv(defcomp100st2vs2equiE2.r,"defcomp100st2vs2equiE2.csv")
save.image(file="defcomp100st2vs2equiE2.rda")
